﻿using AutoMapper;
using BlankApp1.Common;
using BlankApp1.Common.AutoMapper;
using BlankApp1.Common.Extension;
using BlankApp1.ViewModels;
using BlankApp1.Views;
using DryIoc;

using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using NLog.Extensions.Logging;
using PLCTool.ViewModels;
using PLCTool.ViewModels.BasicConfigViewModel;
using PLCTool.ViewModels.BusinessManageViewModel;
using PLCTool.ViewModels.MonitorManageViewModel;
using PLCTool.ViewModels.SystemManageViewModel;
using PLCTool.ViewModels.SystemManageViewModel.Dialogs;
using PLCTool.Views;
using PLCTool.Views.BasicConfigView;
using PLCTool.Views.BusinessManageView;
using PLCTool.Views.MonitorManageView;
using PLCTool.Views.SystemManageView;
using PLCTool.Views.SystemManageView.Dialogs;
using Prism.DryIoc;
using Prism.Ioc;
using Prism.Services.Dialogs;
using SqlSugar;
using System;
using System.Configuration;
using System.Data;
using System.Diagnostics;
using System.IO;
using System.Reflection;
using System.Windows;

namespace BlankApp1
{
    /// <summary>
    /// Interaction logic for App.xaml
    /// </summary>
    public partial class App 
    {
        string comConfig = ConfigurationManager.AppSettings["MySql"];
        protected override Window CreateShell()
        {
            return Container.Resolve<MainWindow>();
        }

        protected override void RegisterTypes(IContainerRegistry containerRegistry)
        {
            var factory = new NLogLoggerFactory();
            ILogger  _logger = factory.CreateLogger("NLog.config");
            containerRegistry.RegisterInstance<ILogger>(_logger);
            containerRegistry.AddRegisterExtension(Assembly.LoadFrom("BizService.dll"), Assembly.LoadFrom("BizService.dll"));//批量注入仓储
            //containerRegistry.Register<IProductRepository, ProductRepository>();
            //containerRegistry.Register<IProductService, ProductService>();
            //注册AutoMapper
            containerRegistry.RegisterSingleton<IAutoMapperProvider, AutoMapperProvider>();
            containerRegistry.Register(typeof(IMapper), GetMapper);
            containerRegistry.RegisterForNavigation<BaseConfigView, BaseConfigViewModel>();
       
         
       
            containerRegistry.RegisterForNavigation<TreeMenuView, TreeMenuViewModel>();
            containerRegistry.RegisterForNavigation<StatisticsView, StatisticsViewModel>();
            containerRegistry.RegisterForNavigation<LogView, LogViewModel>();
            containerRegistry.RegisterForNavigation<PLCPointView, PLCPointViewModel>();
            containerRegistry.RegisterForNavigation<PLCReadView, PLCReadViewModel>();
            containerRegistry.RegisterForNavigation<PLCWriteView, PLCWriteViewModel>();
            containerRegistry.RegisterForNavigation<PLCConfigView, PLCConfigViewModel>();

            containerRegistry.RegisterForNavigation<DeviceKindView, DeviceKindViewModel>();
            containerRegistry.RegisterForNavigation<DeviceView, DeviceViewModel>();
            containerRegistry.RegisterForNavigation<ProjectView, ProjectViewModel>();
            containerRegistry.RegisterForNavigation<ProjectSelectView, ProjectSelectViewModel>();
            containerRegistry.RegisterForNavigation<DeviceTestView, DeviceTestViewModel>();
            containerRegistry.RegisterForNavigation<ResultQueryView, ResultQueryViewModel>();
            containerRegistry.RegisterForNavigation<RetryTestView, RetryTestViewModel>();
            containerRegistry.RegisterForNavigation<AddSchView, AddSchViewModel>();
            containerRegistry.RegisterForNavigation<EditSchView, EditSchViewModel>();
            containerRegistry.RegisterForNavigation<DeviceTestCardView, DeviceTestCardViewModel>();
            containerRegistry.RegisterForNavigation<ProjectTestView, ProjectTestViewModel>();
            containerRegistry.RegisterForNavigation<DeviceResultCardView, DeviceResultCardViewModel>();
            containerRegistry.RegisterForNavigation<ProjectTestResultView, ProjectTestResultViewModel>();
            containerRegistry.RegisterDialog<AddDetailView, AddDetailViewModel>();
            containerRegistry.RegisterDialog<AutoTestView, AutoTestViewModel>();
            containerRegistry.RegisterDialog<ManualTestView, ManualTestViewModel>();
            containerRegistry.RegisterDialog<TestResultDetailView, TestResultDetailViewModel>();
            containerRegistry.RegisterDialog<WritePLCView, WritePLCViewModel>();
            containerRegistry.RegisterDialog<CopySchView, CopySchViewModel>();
            containerRegistry.RegisterDialog<AddOrEditProjectView, AddOrEditProjectViewModel>();
            containerRegistry.RegisterDialog<AddOrEditDeviceKindView, AddOrEditDeviceKindViewModel>();
            containerRegistry.RegisterDialog<AddOrEditDeviceView, AddOrEditDeviceViewModel>();
            containerRegistry.RegisterDialog<EditItemDetailView, EditItemDetailViewModel>();
            containerRegistry.RegisterDialog<ReadSchView, ReadSchViewModel>();
            containerRegistry.RegisterDialog<AddItemDetailView, AddItemDetailViewModel>();
            containerRegistry.RegisterDialog<LoginView, LoginViewModel>();

            containerRegistry.RegisterForNavigation<RoleManageView, RoleManageViewModel>();
            containerRegistry.RegisterForNavigation<MenuManageView, MenuManageViewModel>();
            containerRegistry.RegisterForNavigation<UserManageView, UserManageViewModel>();
            containerRegistry.RegisterDialog<AddEditUserView, AddEditUserViewModel>();
            containerRegistry.RegisterDialog<AddEditRoleView, AddEditRoleViewModel>();
            containerRegistry.RegisterDialog<RoleWithMenuView, RoleWithMenuViewModel>();
            containerRegistry.RegisterDialog<RoleWithProjectView, RoleWithProjectViewModel>();
            containerRegistry.RegisterDialog<AddEditMenuView, AddEditMenuViewModel>();
            containerRegistry.RegisterDialog<UserWithDeviceView, UserWithDeviceViewModel>();
            


        }

        private IMapper GetMapper(IContainerProvider container)
        {
            var provider = container.Resolve<IAutoMapperProvider>();
            return provider.GetMapper();
        }
        protected override void OnInitialized()
        {
            var dialog = Container.Resolve<IDialogService>();
            dialog.ShowDialog("LoginView", callback =>
            {
                if (callback.Result != ButtonResult.OK)
                {
                    Environment.Exit(0);
                    return;
                }
                //调试发现service为MainViewModel
                var service = App.Current.MainWindow.DataContext as IConfigureService;
                if (service != null)
                    service.Configure();
                base.OnInitialized();
            });
         
          
        }
        /// <summary>
        /// 退出登录
        /// </summary>
        /// <param name="containerProvider"></param>
        public static void LoginOut(IContainerProvider containerProvider)
        {
            //Current.MainWindow.Hide();

            //var dialog = containerProvider.Resolve<IDialogService>();

            //dialog.ShowDialog("LoginView", callback =>
            //{
            //    if (callback.Result != ButtonResult.OK)
            //    {
            //        Environment.Exit(0);
            //        return;
            //    }

        
            //});

            // 关闭当前窗口
            Current.MainWindow.Close();
            string exePath=System.AppDomain.CurrentDomain.BaseDirectory + "PLCTool.exe";
            System.Diagnostics.Process.Start(exePath);
            // 获取当前exe文件的路径
          

         
        }

    }
}
