﻿using AutoMapper;
using BizService;
using Model.Dto;
using Model.Entities;
using Prism.Commands;
using Prism.Events;
using Prism.Mvvm;
using Prism.Services.Dialogs;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows;

namespace PLCTool.ViewModels.BasicConfigViewModel
{
    internal class AddDetailViewModel : BindableBase, IDialogAware
    {
        private readonly IEventAggregator _aggregator;
        private readonly IBasicPlcItemConfigService _iBasPlcItemConfigService;

        private readonly IMapper _mapper;
        private List<BasPlcItemConfigDto> plcConfigs = new List<BasPlcItemConfigDto>();
        private long id = 0;
        public AddDetailViewModel(IEventAggregator aggregator, IBasicPlcItemConfigService iBasPlcItemConfigService, IMapper mapper)
        {
            _aggregator = aggregator;
            _iBasPlcItemConfigService=iBasPlcItemConfigService; 
            _mapper = mapper;   
            CloseCommand = new DelegateCommand(Close);
            SureCommand = new DelegateCommand<string>(Sure);
            CancelCommand = new DelegateCommand(Close);
            TxtLostFocusCommand = new DelegateCommand(LostFocus);
            TxtPLCValueLostFocusCommand=new DelegateCommand(LostFocusPLCValue);
        
            PLCTypes.Add("bool");
            PLCTypes.Add("word");
            PLCTypes.Add("dword");
            PLCTypes.Add("real");
            GetPLCConfig();

        }

     




        #region idialog接口实现
        public string Title { set; get; } = "新增PLC变量";

        public event Action<IDialogResult> RequestClose;

        public bool CanCloseDialog()
        {
            return true;
        }

        public void OnDialogClosed()
        {

        }

        public void OnDialogOpened(IDialogParameters parameters)
        {
            var getMsg = parameters.GetValues<BasPlcItemConfigDto>("Key");
            ///值不为空，表示修改
            if (getMsg != null)
            {
                foreach (var item in getMsg)
                {
                    if(item!=null)
                    {
                        Title = "修改PLC变量";
                        PLCItem = item.PlcItem;
                        PLCAddress = item.PlcAddress;
                        SelectPLCType = item.PlcAddType;
                        PLCValue = item.PlcValue;
                        PLCDescribe = item.Remark;

                    }

                }

            }
        }

        #endregion
        #region 私有方法

        private void LostFocusPLCValue()
        {
            if (string.IsNullOrEmpty(PLCValue))
            {
                return;
            }
            if (!IsInteger(PLCValue))
            {
                MessageBox.Show("请填写正整数！", "提示", MessageBoxButton.OK, MessageBoxImage.Warning);
                return;
            }
        }
        /// <summary>
        /// 判断输入的数据
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        private bool IsInteger(string input)
        {
            Regex regex = new Regex(@"^[0-9]\d*$");
            return regex.IsMatch(input);
        }
        private void LostFocus()
        {
            var findResult = plcConfigs?.FirstOrDefault(x => x.PlcAddress == PLCAddress);
            if (findResult != null)
            {
                id = findResult.Id;
                PLCItem = findResult.PlcItem;
                SelectPLCType = findResult.PlcAddType;
                PLCValue = findResult.PlcValue;
                PLCDescribe = findResult.Remark;
            }

        }
        private void Close()
        {
            RequestClose?.Invoke(new DialogResult(ButtonResult.Cancel));
        }
        /// <summary>
        /// 确认
        /// </summary>
        /// <param name="obj"></param>
        private void Sure(string obj)
        {
            if ( string.IsNullOrEmpty(PLCAddress))
            {
                MessageBox.Show("请填写PLC地址！", "提示", MessageBoxButton.OK, MessageBoxImage.Information);
                return;
            }
            if (string.IsNullOrEmpty(PLCItem))
            {
                MessageBox.Show("请填写PLC变量名！", "提示", MessageBoxButton.OK, MessageBoxImage.Information);
                return;
            }
            if (string.IsNullOrEmpty(SelectPLCType) )
            {
                MessageBox.Show("请选择PLC变量类型！", "提示", MessageBoxButton.OK, MessageBoxImage.Information);
                return;
            }
            if(SelectPLCType=="bool")
            {
                if((PLCValue!="0")&&(PLCValue!="1"))
                {
                    MessageBox.Show("PLC变量类型为bool，数据值必须为0或者1！", "提示", MessageBoxButton.OK, MessageBoxImage.Information);
                    return;
                }
               
            }
            BasPlcItemConfigDto basPlcItemConfigDto = new BasPlcItemConfigDto()
            {
                Id = id,
                PlcItem =PLCItem,
                PlcAddress=PLCAddress,
                PlcAddType =SelectPLCType ,
                PlcValue =PLCValue,
                Remark=PLCDescribe
                
            };
            DialogParameters parm = new DialogParameters();
            parm.Add("ReturnValue", basPlcItemConfigDto);
            RequestClose?.Invoke(new DialogResult(ButtonResult.OK, parm));
        }
        /// <summary>
        /// 重置
        /// </summary>
        /// <param name="obj"></param>
        private void ResetMethod(string obj)
        {

        }
        /// <summary>
        /// 获取PLC配置
        /// </summary>
        private void GetPLCConfig()
        {
            var plclist = _iBasPlcItemConfigService.QueryList();

            plcConfigs = _mapper.Map<List<bas_plc_item_config>, List<BasPlcItemConfigDto>>(plclist);
        }
        #endregion



        #region 命令绑定
        public DelegateCommand CloseCommand { set; get; }
        public DelegateCommand<string> SureCommand { set; get; }
        public DelegateCommand CancelCommand { set; get; }
        public DelegateCommand TxtLostFocusCommand { set; get; }
        public DelegateCommand TxtPLCValueLostFocusCommand { set; get; }
        

        #endregion
        #region 变量绑定
        private string plcAddress;
        public string PLCAddress
        {
            get { return plcAddress; }
            set { plcAddress = value; RaisePropertyChanged(); }
        }

        private  List<string> plcVars;
        public List<string> PLCVars
        {
            get { return plcVars; }
            set { plcVars = value; RaisePropertyChanged(); }
        }
        private string selectPLCVar;
        public string SelectPLCVar
        {
            get { return selectPLCVar; }
            set { selectPLCVar = value; RaisePropertyChanged(); }
        }

        private string plcItem;
        public string PLCItem
        {
            get { return plcItem; }
            set { plcItem = value; RaisePropertyChanged(); }
        }
        private string selectPLCType;
        public string SelectPLCType
        {
            get { return selectPLCType; }
            set { selectPLCType = value; RaisePropertyChanged(); }
        }
        /// <summary>
        /// 逻辑判断
        /// </summary>
        private List<string> plcTypes = new List<string>();
        public List<string> PLCTypes
        {
            get { return plcTypes; }
            set { plcTypes = value; RaisePropertyChanged(); }
        }
        private string plcValue;
        public string PLCValue
        {
            get { return plcValue; }
            set { plcValue = value; RaisePropertyChanged(); }
        }
        private string plcDescribe;
        public string PLCDescribe
        {
            get { return plcDescribe; }
            set { plcDescribe = value; RaisePropertyChanged(); }
        }


        #endregion
    }
}
