﻿using AutoMapper;
using AutoUpdaterDotNET;
using BizService;
using BlankApp1.Common;
using BlankApp1.Events;
using BlankApp1.Views;
using Microsoft.Extensions.Logging;
using Model.Dto;
using PLCTool.Common;
using PLCTool.Events;
using PLCTool.Models;
using Prism.Commands;
using Prism.Events;
using Prism.Ioc;
using Prism.Mvvm;
using Prism.Regions;

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Configuration;
using System.IO;
using System.Reflection;
using System.Threading.Tasks;
using System.Windows;

namespace BlankApp1.ViewModels
{
    public class MainWindowViewModel : BindableBase, IConfigureService
    {
        private readonly ILogger _logger;
        private readonly IRegionManager _regionManager;
        private readonly IMapper _mapper;
        private readonly IContainerProvider _containerProvider;
        private string _title = "Prism Application";
        private IRegionNavigationJournal journal;
        private  string autoUpdateServer = ConfigurationManager.AppSettings["AutoUpdateServer"];
        
        public string Title
        {
            get { return _title; }
            set { SetProperty(ref _title, value); }
        }

        public MainWindowViewModel(IRegionManager regionManager,ILogger logger, IEventAggregator aggregator, IMapper mapper, IContainerProvider containerProvider)
        {
            this._regionManager = regionManager;
            this._logger = logger;
            this._mapper = mapper;
            this._containerProvider = containerProvider;
            ButtonNavigateCommand = new DelegateCommand<string>(ButtonNavigate);
            BackCommand = new DelegateCommand(GoBack);
            NextCommand = new DelegateCommand(GoNext);
            BreadChangedCommand = new DelegateCommand<object>(BreadChangedC);
            LoginOutCommand = new DelegateCommand<object>(LoginOut);
            OnLoadCommand = new DelegateCommand(OnLoad);
            VersisonCommand = new DelegateCommand(UpdateVersion);
            JudgePLC();
            logger.LogInformation("info");
            logger.LogError("error");

            aggregator.GetEvent<BreadEvent>().Subscribe(RecvBreadMsg);
            aggregator.GetEvent<MaskEvent>().Subscribe(DisplayMask);
            //检查更新版本
            AutoUpdater.Start($"http://{autoUpdateServer}/Updates/AutoUpdaterStarter.xml");

        }

        private void UpdateVersion()
        {
            AutoUpdater.Start($"http://{autoUpdateServer}/Updates/AutoUpdaterStarter.xml");
        }

        private void OnLoad()
        {
  
        }

        private void LoginOut(object obj)
        {
           
            App.LoginOut(_containerProvider);
        }

        private void DisplayMask(bool isVis)
        {
            IsMaskVisible=isVis;
        }

        private void RecvBreadMsg(List<CrumbViewModel> list)
        {
            Path.Clear();
            foreach (var item in list)
            {
                Path.Add(item);
            }
        }
        /// <summary>
        /// 面包靴导航
        /// </summary>
        /// <param name="obj"></param>
        private void BreadChangedC(object obj)
        {
            if(!string.IsNullOrEmpty(SelectedCrumb?.NavigateRegion))
            {
                _regionManager.Regions["ContentRegion"].RequestNavigate(SelectedCrumb.NavigateRegion);
            }
      
        }

        private void GoNext()
        {
            if(journal.CanGoForward)
                journal.GoForward();    
        }

        private void GoBack()
        {
            if(journal.CanGoBack)
                journal.GoBack();
        }

        private void ButtonNavigate(string strPar)
        {
            if (strPar == null || string.IsNullOrWhiteSpace(strPar))
            {
                return;
            }
            //导航页面
            _regionManager.Regions["ContentRegion"].RequestNavigate(strPar,arg=>
            {
                journal=arg.Context.NavigationService.Journal;
            });
        }

        /// <summary>
        /// 配置首页初始化参数，这个Configure是实现的接口IConfigureService中定义的方法；
        /// </summary>
        public void Configure()
        {
            UserName = Appsession.UserName;
            //默认进入简报页面
            _regionManager.Regions["ContentRegion"].RequestNavigate("LogView");
            _regionManager.Regions["MenuRegion"].RequestNavigate("TreeMenuView");
            _regionManager.Regions["ContentRegion"].RequestNavigate("StatisticsView");
         
        }
        /// <summary>
        /// 连接PLC
        /// </summary>
        private  void JudgePLC()
        {
            
                Task.Run(async () =>
                {
                    while (true)
                    {
                       
                        //更新界面上的值
                        Application.Current.Dispatcher.Invoke(async () =>
                        {
                            PLCIsConnect =await PLCConnectionFactory.CreatePLCCont().IsConnectPLC();

                        });

                        await Task.Delay(5000);
                    }

                });
            
        }
        #region 命令绑定
        public DelegateCommand<string> ButtonNavigateCommand { set; get; }
        public DelegateCommand NextCommand { set; get; }
        public DelegateCommand BackCommand { set; get; }
        public DelegateCommand OnLoadCommand { set; get; }
        public DelegateCommand<object> BreadChangedCommand { set; get; }
        public DelegateCommand<object> LoginOutCommand { set; get; }

        public DelegateCommand VersisonCommand { set; get; }
        

        #endregion
        #region 数据绑定
        /// <summary>
        /// 面包削
        /// </summary>
        private ObservableCollection<CrumbViewModel> _path = new ObservableCollection<CrumbViewModel>();
        public ObservableCollection<CrumbViewModel> Path
        {
            get { return _path; }
            set { SetProperty(ref _path, value); }
        }
        private CrumbViewModel selectedCrumb = new CrumbViewModel();
        public CrumbViewModel SelectedCrumb
        {
            get { return selectedCrumb; }
            set { SetProperty(ref selectedCrumb, value); }
        }

        private bool plcIsConnect = false;
        public bool PLCIsConnect
        {
            get { return plcIsConnect; }
            set { plcIsConnect = value; RaisePropertyChanged(); }
        }

        private bool isMaskVisible = false;
        public bool IsMaskVisible
        {
            get { return isMaskVisible; }
            set { isMaskVisible = value; RaisePropertyChanged(); }
        }

        private string userName ;
        public string UserName
        {
            get { return userName; }
            set { userName = value; RaisePropertyChanged(); }
        }
        private string? version = Assembly.GetExecutingAssembly()?.GetName()?.Version?.ToString();
        public string Version
        {
            get { return $"版本：V{version}"; }
            set { version = value; RaisePropertyChanged(); }
        }

        #endregion
    }
}
