﻿using AutoMapper;
using BizService;
using Microsoft.Extensions.Logging;
using Model.Dto;
using Model.Entities;
using PLCTool.Common;
using PLCTool.Controls;
using Prism.Commands;
using Prism.Mvvm;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows;

namespace PLCTool.ViewModels.MonitorManageViewModel
{
    public  class PLCWriteViewModel:BindableBase
    {
        private readonly IBasicPlcItemConfigService _iBasPlcItemConfigService;
        private readonly ILogger _logger;
        private readonly IMapper _mapper;
        private List<BasPlcItemConfigDto> plcConfigs = new List<BasPlcItemConfigDto>();
        public PLCWriteViewModel(IBasicPlcItemConfigService iBasPlcItemConfigService, IMapper mapper,ILogger logger)
        {

            _iBasPlcItemConfigService = iBasPlcItemConfigService;
            _mapper = mapper;
            _logger= logger;
            WriteCommand = new DelegateCommand<object>(WriteToPlc);
            OnLoadCommand = new DelegateCommand(OnLoad);
            GetPLCConfig();
        }


        #region 私有方法
        private void OnLoad()
        {
            GetPLCConfig();
        }
        private async void WriteToPlc(object obj)
        {//没有连接 plc，直接退出
            if (!await PLCConnectionFactory.CreatePLCCont().IsConnectPLC())
            {
                MessageBox.Show("PLC未连接！", "提示", MessageBoxButton.OK, MessageBoxImage.Warning);
                return;
            }

            long id = Convert.ToInt64(obj);
            var findPlcItem= PLCItemList.FirstOrDefault(l => l.Id == id);
            if(findPlcItem!=null)
            {
                string plcAddr = findPlcItem.PlcAddress;
                string plcAddType = findPlcItem.PlcAddType;
                string plcValue = findPlcItem.PlcValue;
                if (string.IsNullOrEmpty(plcValue))
                {
                    MessageBox.Show("输入为空，请填写数据！", "提示", MessageBoxButton.OK, MessageBoxImage.Warning);
                    return;
                }
                if (!IsInteger(plcValue))
                {
                    MessageBox.Show("请填写正整数！", "提示", MessageBoxButton.OK, MessageBoxImage.Warning);
                    return;
                }
                switch (plcAddType)
                {
                    case "bool":
                        bool isResult = PLCConnectionFactory.CreatePLCCont().WritePlcObject(plcAddr, VarType.Bit, plcValue);
                        if(isResult)
                        {
                            _logger.LogInformation($"向PLC写入值。plc地址{plcAddr}，数据值{plcValue}");
                            MessageBox.Show("写入成功！", "提示", MessageBoxButton.OK, MessageBoxImage.Information);
                        }
                        break;
                    case "word":
                        bool isResult2 = PLCConnectionFactory.CreatePLCCont().WritePlcObject(plcAddr, VarType.Word, plcValue);
                        if (isResult2)
                        {
                            _logger.LogInformation($"向PLC写入值。plc地址{plcAddr}，数据值{plcValue}");
                            MessageBox.Show("写入成功！", "提示", MessageBoxButton.OK, MessageBoxImage.Information);
                        }
                        break;
                    case "dword":
                        bool isResult3 = PLCConnectionFactory.CreatePLCCont().WritePlcObject(plcAddr, VarType.DWord, plcValue);
                        if (isResult3)
                        {
                            _logger.LogInformation($"向PLC写入值。plc地址{plcAddr}，数据值{plcValue}");
                            MessageBox.Show("写入成功！", "提示", MessageBoxButton.OK, MessageBoxImage.Information);
                        }
                        break;
                    case "real":
                       
                        bool isResult4 = PLCConnectionFactory.CreatePLCCont().WritePlcObject(plcAddr, VarType.Real, plcValue);
                        if (isResult4)
                        {
                            _logger.LogInformation($"向PLC写入值。plc地址{plcAddr}，数据值{plcValue}");
                            MessageBox.Show("写入成功！", "提示", MessageBoxButton.OK, MessageBoxImage.Information);
                        }
                        break;
                }

            }
        
        }
        /// <summary>
        /// 判断输入的数据
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        private  bool IsInteger(string input)
        {
            Regex regex = new Regex(@"^[0-9]\d*$");
            return regex.IsMatch(input);
        }
        /// <summary>
        /// 获取PLC配置
        /// </summary>
        private void GetPLCConfig()
        {
            var plclist = _iBasPlcItemConfigService.QueryList();
            plcConfigs = _mapper.Map<List<bas_plc_item_config>, List<BasPlcItemConfigDto>>(plclist);
            foreach (var plc in plcConfigs)
            {
                var find = PLCItemList.FirstOrDefault(X => X.PlcAddress == plc.PlcAddress);
                if (find == null)
                {
                    PLCItemList.Add(plc);
                }
            }
            PLCItemList = new AsyncObservableCollection<BasPlcItemConfigDto>(PLCItemList.OrderBy(x => x.Id));
        }

    
        #endregion
        #region 数据绑定
        private AsyncObservableCollection<BasPlcItemConfigDto> plcItemList = new AsyncObservableCollection<BasPlcItemConfigDto>();
        public AsyncObservableCollection<BasPlcItemConfigDto> PLCItemList
        {
            get { return plcItemList; }
            set { plcItemList = value; RaisePropertyChanged(); }
        }


        #endregion

        #region 命令绑定
        public DelegateCommand OnLoadCommand { set; get; }
        public DelegateCommand<object> WriteCommand { set; get; }
        #endregion
    }
}

