﻿using AutoMapper;
using BizService;
using Model.Dto;
using Model.Entities;
using PLCTool.Models;
using Prism.Commands;
using Prism.Events;
using Prism.Mvvm;
using Prism.Services.Dialogs;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Forms;

namespace PLCTool.ViewModels.SystemManageViewModel.Dialogs
{
 
    public class AddEditMenuViewModel : BindableBase, IDialogAware
    {
        private readonly IEventAggregator _aggregator;
  
        private readonly IMenuService _iMenuService;
        private readonly IMapper _mapper;
        private List<MenuDto> allMenuInfoList = new List<MenuDto>();
        private string basePath = System.IO.Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Images");
        private string initMenuHeader;
        private int id = 0;
        public AddEditMenuViewModel(IEventAggregator aggregator, IMenuService iMenuService, IMapper mapper)
        {
            _aggregator = aggregator;
            
            _iMenuService = iMenuService;
            _mapper = mapper;
            CloseCommand = new DelegateCommand(Close);
            SureCommand = new DelegateCommand<string>(Sure);
            CancelCommand = new DelegateCommand(Close);
            ChangeImageCommand = new DelegateCommand(ChangeImage);
         
            UpCommand = new DelegateCommand<object>(DataUp);
            DownCommand = new DelegateCommand<object>(DataDown);
            GetMenus();
          

        }

        private void DataDown(object obj)
        {
            {
                int id = Convert.ToInt32(obj);
                var findScheExist = MenuList.FirstOrDefault(x => x.Index == id);
                if (findScheExist != null)
                {

                    int index = MenuList.IndexOf(findScheExist);
                    if (index != -1)
                    {
                        if (index == MenuList.Count - 1)
                            return;
                        MenuList.RemoveAt(index);
                        index = index + 1;
                        MenuList.Insert(index, findScheExist);
                        //现在d的排序
                        IndexOrder = MenuList.ToList().FindIndex(x => x.MenuHeader == initMenuHeader) + 1;
                    }


                }
            }
        }

        private void DataUp(object obj)
        {
            int id = Convert.ToInt32(obj);
            var findScheExist = MenuList.FirstOrDefault(x => x.Index == id);
            if (findScheExist != null)
            {

                int index = MenuList.IndexOf(findScheExist);
                if (index != -1)
                {
                    if (index == 0)
                        return;
                    MenuList.RemoveAt(index);
                    index = index - 1;
                    MenuList.Insert(index, findScheExist);
                    //现在d的排序
                    IndexOrder = MenuList.ToList().FindIndex(x => x.MenuHeader == initMenuHeader)+1;
                }


            }

        }

        private void ChangeOrder()
        {
       
        }



        #region idialog接口实现
        public string Title { set; get; } = "新增菜单";

        public event Action<IDialogResult> RequestClose;

        public bool CanCloseDialog()
        {
            return true;
        }

        public void OnDialogClosed()
        {

        }

        public void OnDialogOpened(IDialogParameters parameters)
        {
            var getMsg = parameters.GetValues<int>("Key");
            ///值不为空，表示修改
            if (getMsg != null)
            {
                foreach (var item in getMsg)
                {
                    if (item != null)
                    {
                        Title = "修改菜单";
                        id = item;
                        var findMenu = allMenuInfoList.FirstOrDefault(x => x.MenuId == id);
                        if (findMenu != null)
                        {
                            //防止改名后找不到
                            initMenuHeader = findMenu.MenuHeader;
                            MenuHeader = findMenu.MenuHeader;
                            TargetView = findMenu.TargetView;
                            ParentId = findMenu.ParentId;
                            if (!string.IsNullOrEmpty(findMenu.MenuIcon))
                            {
                                IsChangeImg = true;
                                MenuIcon = Path.Combine(basePath, findMenu.MenuIcon);
                            }
                            else
                            {
                                IsChangeImg = false;
                            }
                          
                            IndexOrder = findMenu.Index;
                            //查找这个ParentId下的所有菜单
                            var findMenus=_iMenuService.FindAllByParentId(ParentId).OrderBy(x=>x.Index);
                            foreach(var each in  findMenus)
                            {
                                MenuList.Add(new MenuDto
                                {
                                    MenuHeader= each.MenuHeader,
                                    Index=each.Index,
                                });
                            }
                        }

                    }

                }

            }
        }

        #endregion
        #region 私有方法

        /// <summary>
        /// 更改ico图片
        /// </summary>
        private void ChangeImage()
        {
            using (OpenFileDialog openFileDialog = new OpenFileDialog())
            {
                openFileDialog.Title = "选择图标文件";
                openFileDialog.Filter = "图像文件 (*.png)|*.png";

                if (openFileDialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    try {
                        // 用户选择了文件
                        string filePath = openFileDialog.FileName;
                        string fileName = Path.GetFileName(filePath);
                        string directoryPath = Path.GetDirectoryName(filePath);
                        string destinationPath = Path.Combine(basePath, fileName);
                        //就是选择当前文件下的图片，就不复制图片了
                        if (directoryPath == basePath)
                        {
                            MenuIcon = destinationPath;
                        }
                        else
                        {
                            // 检查目标文件夹中是否已存在同名文件
                            if (File.Exists(destinationPath))
                            {
                                // 如果存在，可以选择覆盖或抛出异常
                                System.Windows.MessageBox.Show("已存在此图片名！", "提示", MessageBoxButton.OK, MessageBoxImage.Information);
                                return;
                            }
                            else
                            {
                                // 如果不存在，则复制文件
                                File.Copy(filePath, destinationPath);
                                MenuIcon = destinationPath;

                            }
                        }
                    }
                    catch(Exception ex) 
                    { 

                    }
                 

                        
               
                }
            }
        }





        private void Close()
        {
            RequestClose?.Invoke(new Prism.Services.Dialogs.DialogResult(ButtonResult.Cancel));
        }
        /// <summary>
        /// 确认
        /// </summary>
        /// <param name="obj"></param>
        private void Sure(string obj)
        {
            if (string.IsNullOrEmpty(MenuHeader))
            {
                System.Windows.MessageBox.Show("请填写菜单名", "提示", MessageBoxButton.OK, MessageBoxImage.Information);
                return;
            }
            if (string.IsNullOrEmpty(TargetView))
            {
                System.Windows.MessageBox.Show("请填写菜单界面！", "提示", MessageBoxButton.OK, MessageBoxImage.Information);
                return;
            }
            //if (string.IsNullOrEmpty(MenuIcon))
            //{
            //    System.Windows.MessageBox.Show("请选择图标！", "提示", MessageBoxButton.OK, MessageBoxImage.Information);
            //    return;
            //}
          
            //判断是否存在用户名

            var findResult = allMenuInfoList.FirstOrDefault(x => x.MenuHeader == MenuHeader);
            if (findResult != null)
            {
                if (Title == "新增菜单")
                {
                    System.Windows.MessageBox.Show("已存在此菜单名！", "提示", MessageBoxButton.OK, MessageBoxImage.Information);
                    return;
                }
                else
                {
                    //修改时,若查到其他的用户名和这个重复
                    if (findResult.MenuId != id)
                    {
                        System.Windows.MessageBox.Show("已存在此菜单名！", "提示", MessageBoxButton.OK, MessageBoxImage.Information);
                        return;
                    }
                }
            }


            if (Title == "新增菜单")
            {
                AddMenu();
            }
            else
            {
                EditMenu();

            }

            RequestClose?.Invoke(new Prism.Services.Dialogs.DialogResult(ButtonResult.OK));
        }

        private void AddMenu()
        {

            ////添加到数据库user表中
            //iMenuService.Add(new User()
            //{
            //    UserName = UserName,
            //    Password = PassWord,
            //    Description = Describe
            //});
            ////查找添加的userid
            //int newUserId = 0;
            //var findId = _iUserService.QueryList().FirstOrDefault(x => x.UserName == UserName);
            //if (findId != null)
            //{
            //    newUserId = findId.Id;
            //}
            ////添加到数据库useRole表中
            //bool isAdd = _iMenuService.Add(new UserRole()
            //{
            //    UserId = newUserId,
            //    RoleId = allRoles.FirstOrDefault(x => x.Name == (SelectRole)).Id
            //});
            //if (isAdd)
            //{
            //    MessageBox.Show("添加用户成功", "提示", MessageBoxButton.OK, MessageBoxImage.Information);

            //}

        }
        private void EditMenu()
        {
            Menu menu = new Menu()
            {
                MenuId= id,
                MenuHeader = MenuHeader,
                TargetView = TargetView,
                ParentId = ParentId,
                MenuIcon= Path.GetFileName(MenuIcon),
                Index =IndexOrder

            };
            //修改到数据库user表中
            bool isedit = _iMenuService.Edit(menu);
            //修改其他的排序编号
            var otherMenus = _iMenuService.FindAllByParentId(ParentId);
            foreach(var otherMenu in otherMenus )
            {
                if(otherMenu.MenuId == id )
                {
                    continue;
                }
                //更新排序
                otherMenu.Index = MenuList.ToList().FindIndex(x => x.MenuHeader == otherMenu.MenuHeader) + 1; ;
              
                _iMenuService.Edit(otherMenu);
              
            }
            if (isedit)
            {
                System.Windows.MessageBox.Show("修改菜单成功", "提示", MessageBoxButton.OK, MessageBoxImage.Information);

            }
           


        }
   

        private void GetMenus()
        {
            var menus = _iMenuService.QueryList();
            allMenuInfoList = _mapper.Map<List<Menu>, List<MenuDto>>(menus);
        }
        #endregion



        #region 命令绑定
        public DelegateCommand CloseCommand { set; get; }
        public DelegateCommand<string> SureCommand { set; get; }
        public DelegateCommand CancelCommand { set; get; }

        public DelegateCommand ChangeImageCommand { set; get; }
        public DelegateCommand ChangeOrderCommand { set; get; }

        public DelegateCommand<object> DownCommand { set; get; }
        public DelegateCommand<object> UpCommand { set; get; }
        #endregion
        #region 变量绑定
        private string menuHeader;
        public string MenuHeader
        {
            get { return menuHeader; }
            set { menuHeader = value; RaisePropertyChanged(); }
        }
        private string targetView;
        public string TargetView
        {
            get { return targetView; }
            set { targetView = value; RaisePropertyChanged(); }
        }

        private int parentId;
        public int ParentId
        {
            get { return parentId; }
            set { parentId = value; RaisePropertyChanged(); }
        }

        private string menuIcon;
        public string MenuIcon
        {
            get { return menuIcon; }
            set { menuIcon = value; RaisePropertyChanged(); }
        }
        private int indexOrder;
        public int IndexOrder
        {
            get { return indexOrder; }
            set { indexOrder = value; RaisePropertyChanged(); }
        }
        
     

        private string describe;
        public string Describe
        {
            get { return describe; }
            set { describe = value; RaisePropertyChanged(); }
        }
        private bool isChangeImg=true;
        public bool IsChangeImg
        {
            get { return isChangeImg; }
            set { isChangeImg = value; RaisePropertyChanged(); }
        }

        private ObservableCollection<MenuDto> menuList = new ObservableCollection<MenuDto>();
        public ObservableCollection<MenuDto> MenuList
        {
            get { return menuList; }
            set { SetProperty(ref menuList, value); }
        }

        #endregion
    }
}
