﻿using AutoMapper;
using BizService;
using Model.Dto;
using Model.Entities;
using Prism.Commands;
using Prism.Events;
using Prism.Mvvm;
using Prism.Services.Dialogs;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using static System.Windows.Forms.VisualStyles.VisualStyleElement.StartPanel;

namespace PLCTool.ViewModels.SystemManageViewModel.Dialogs
{
  
    public class AddEditRoleViewModel : BindableBase, IDialogAware
    {
        private readonly IEventAggregator _aggregator;
        private readonly IUserService _iUserService;
        private readonly IRoleService _iRoleService;
        private readonly IUserRoleService _iUserRoleService;
        private readonly IMapper _mapper;

        private List<Role> allRoles = new List<Role>();

        private int id = 0;
        public AddEditRoleViewModel(IEventAggregator aggregator, IUserService iUserService, IRoleService iRoleService, IUserRoleService iUserRoleService, IMapper mapper)
        {
            _aggregator = aggregator;
            _iUserService = iUserService;
            _iRoleService = iRoleService;
            _iUserRoleService = iUserRoleService;
            _mapper = mapper;
            CloseCommand = new DelegateCommand(Close);
            SureCommand = new DelegateCommand<string>(Sure);
            CancelCommand = new DelegateCommand(Close);

            GetRoles();

        }




        #region idialog接口实现
        public string Title { set; get; } = "新增角色";

        public event Action<IDialogResult> RequestClose;

        public bool CanCloseDialog()
        {
            return true;
        }

        public void OnDialogClosed()
        {

        }

        public void OnDialogOpened(IDialogParameters parameters)
        {
            var getMsg = parameters.GetValues<int>("Key");
            ///值不为空，表示修改
            if (getMsg != null)
            {
                foreach (var item in getMsg)
                {
                    if (item != null)
                    {
                        Title = "修改角色";
                        id = item;
                        var findUser = allRoles.FirstOrDefault(x => x.Id == id);
                        if (findUser != null)
                        {
                            RoleName = findUser.Name;
                           
                            Describe = findUser.Description;
                        }
                    
                    }

                }

            }
        }

        #endregion
        #region 私有方法


        /// <summary>
        /// 判断输入的数据
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>


        private void Close()
        {
            RequestClose?.Invoke(new DialogResult(ButtonResult.Cancel));
        }
        /// <summary>
        /// 确认
        /// </summary>
        /// <param name="obj"></param>
        private void Sure(string obj)
        {
            if (string.IsNullOrEmpty(RoleName))
            {
                MessageBox.Show("请填写角色名", "提示", MessageBoxButton.OK, MessageBoxImage.Information);
                return;
            }
        
          
            //判断是否存在用户名

            var findResult = allRoles.FirstOrDefault(x => x.Name == RoleName);
            if (findResult != null)
            {
                if (Title == "新增角色")
                {
                    MessageBox.Show("已存在此角色名！", "提示", MessageBoxButton.OK, MessageBoxImage.Information);
                    return;
                }
                else
                {
                    //修改时,若查到其他的用户名和这个重复
                    if (findResult.Id != id)
                    {
                        MessageBox.Show("已存在此角色名！", "提示", MessageBoxButton.OK, MessageBoxImage.Information);
                        return;
                    }
                }
            }


            if (Title == "新增角色")
            {
                bool isAdd=_iRoleService.Add(new Role()
                {
                    Name=RoleName,
                    Description=Describe

                });
                if (isAdd)
                {
                    MessageBox.Show("添加角色成功", "提示", MessageBoxButton.OK, MessageBoxImage.Information);

                }
            }
            else
            {

                bool isEdit = _iRoleService.Edit(new Role()
                {
                    Id = id,
                    Name = RoleName,
                    Description = Describe

                });
                if (isEdit)
                {
                    MessageBox.Show("修改角色成功", "提示", MessageBoxButton.OK, MessageBoxImage.Information);

                }
            }

            RequestClose?.Invoke(new DialogResult(ButtonResult.OK));
        }

     
      
        /// <summary>
        /// 获取角色
        /// </summary>
        private void GetRoles()
        {
            allRoles = _iRoleService.QueryList();

           
        }

    
        #endregion



        #region 命令绑定
        public DelegateCommand CloseCommand { set; get; }
        public DelegateCommand<string> SureCommand { set; get; }
        public DelegateCommand CancelCommand { set; get; }


        #endregion
        #region 变量绑定
        private string roleName;
        public string RoleName
        {
            get { return roleName; }
            set { roleName = value; RaisePropertyChanged(); }
        }
      

        private string describe;
        public string Describe
        {
            get { return describe; }
            set { describe = value; RaisePropertyChanged(); }
        }

        #endregion
    }
}
