﻿using AutoMapper;
using BizService;
using Microsoft.Extensions.Logging;
using Model.Entities;
using PLCTool.Models;
using Prism.Commands;
using Prism.Events;
using Prism.Mvvm;
using Prism.Services.Dialogs;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace PLCTool.ViewModels.SystemManageViewModel.Dialogs
{
    
    public class RoleWithProjectViewModel : BindableBase, IDialogAware
    {
        private readonly IMapper _mapper;
        private readonly ILogger _logger;
        private readonly IRoleService _iRoleService;
        
        private readonly IRoleProjectService _iRoleProjectService;
        private readonly IBasicProjectService _iBasicProjectService;
        private string tyepName = String.Empty;
        private int roleId;

        public RoleWithProjectViewModel(IEventAggregator aggregator, IRoleService iRoleService, IRoleProjectService iRoleProjectService, IBasicProjectService iBasicProjectService, IMapper mapper, ILogger logger)
        {


            _mapper = mapper;
            _logger = logger;
            _iRoleProjectService =iRoleProjectService;
            _iBasicProjectService = iBasicProjectService;
            _iRoleService = iRoleService;
            SureCommand = new DelegateCommand<string>(Sure);

            SelectAllCommand = new DelegateCommand<object>(SelectAll);
            UnSelectAllCommand = new DelegateCommand<object>(UnSelecttAll);

        }

        private void GetAllProjects()
        {
            var projects = _iBasicProjectService.QueryList();
            foreach (var project in projects)
            {
                //查找角色是否带有这个项目
                bool isSelect = false;

                var findMenu = _iRoleProjectService.FindAllByRoleId(roleId).FirstOrDefault(x => x.ProjectId == project.project_id);
                if (findMenu != null)
                {
                    isSelect = true;
                }
                RoleWithProjectList.Add(new SelectRoleProjectModel
                {
                    RoleId = roleId,
                    ProjectName = project.project_name,
                    ProjectId=(int)project.project_id,
                    Describe = project.remark,
                    IsSelected = isSelect
                });
            }
        }

        private void UnSelecttAll(object obj)
        {
            RoleWithProjectList.ToList().FindAll(p => p.IsSelected = false);
        }

        private void SelectAll(object obj)
        {
            RoleWithProjectList.ToList().FindAll(p => p.IsSelected = true);
        }




        private void Sure(string obj)
        {
            //查找是否有这个测试方案

            var findSelect = RoleWithProjectList.FirstOrDefault(x => x.IsSelected == true);
            if (findSelect == null)
            {
                MessageBoxResult boxResult = MessageBox.Show("请勾选测试项！", "提示", MessageBoxButton.OK, MessageBoxImage.Warning);
                return;
            }
            //查找的信息
            //删除这个角色以前的菜单
            var findPros = _iRoleProjectService.FindAllByRoleId(roleId);
            foreach (var item in findPros)
            {
                _iRoleProjectService.Delete(item.Id);
            }
            //添加角色关联的菜单到数据库
            var roleProjectss = RoleWithProjectList.ToList().FindAll(x => x.IsSelected).OrderBy(y => y.ProjectId);
            foreach (var item in roleProjectss)
            {
                _iRoleProjectService.Add(new RoleProject
                {
                    RoleId = roleId,
                    ProjectId = item.ProjectId,
                });
            }
            RequestClose?.Invoke(new DialogResult(ButtonResult.OK));
        }

        public string Title { set; get; } = "授权角色项目";

        public event Action<IDialogResult> RequestClose;

        public bool CanCloseDialog()
        {
            return true;
        }

        public void OnDialogClosed()
        {

        }

        public void OnDialogOpened(IDialogParameters parameters)
        {

            //设备类型
            var getMsg = parameters.GetValues<int>("Key");

            if (getMsg != null)
            {

                foreach (var item in getMsg)
                {

                    roleId = item;
                    GetAllProjects();
                    RoleName = _iRoleService.Find(roleId).Name;
                }


            }


        }
        #region 命令绑定
        public DelegateCommand<string> SureCommand { set; get; }


        public DelegateCommand<object> SelectAllCommand { set; get; }
        public DelegateCommand<object> UnSelectAllCommand { set; get; }

        #endregion
        #region 数据绑定
        /// <summary>
        /// 测试方案编码
        /// </summary>
        private string roleName;
        public string RoleName
        {
            get { return roleName; }
            set { roleName = value; RaisePropertyChanged(); }
        }



        private ObservableCollection<SelectRoleProjectModel> roleWithProjectList = new ObservableCollection<SelectRoleProjectModel>();
        public ObservableCollection<SelectRoleProjectModel> RoleWithProjectList
        {
            get { return roleWithProjectList; }
            set { SetProperty(ref roleWithProjectList, value); }
        }


        #endregion

    }
}
