﻿using AutoMapper;
using BizService;
using Microsoft.Extensions.Logging;
using Model.Dto;
using Model.Entities;
using PLCTool.Events;
using PLCTool.Models;
using Prism.Commands;
using Prism.Events;
using Prism.Mvvm;
using Prism.Services.Dialogs;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace PLCTool.ViewModels.SystemManageViewModel
{
    
    public class MenuManageViewModel : BindableBase
    {
        private readonly IMenuService _iMenuService;
        private readonly ILogger _logger;
        private readonly IMapper _mapper;
        private readonly IDialogService _dialog;
        private readonly IEventAggregator _aggregator;
        private List<MenuDto> allMenuInfoList = new List<MenuDto>();
        private string basePath=System.IO.Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Images");
        public MenuManageViewModel(IMenuService iMenuService, IMapper mapper, ILogger logger, IDialogService dialog, IEventAggregator aggregator)
        {

            _iMenuService = iMenuService;
            _mapper = mapper;
            _logger = logger;
            _dialog = dialog;
            _aggregator = aggregator;
            AddCommand = new DelegateCommand<object>(AddMenu);
            EditCommand = new DelegateCommand<object>(EditMenu);
            DeleteCommand = new DelegateCommand<object>(Delete);
            OnLoadCommand = new DelegateCommand(OnLoad);
            QueryCommand = new DelegateCommand<object>(Query);
        }




        #region 私有方法
        private void OnLoad()
        {
            GetAllMenus();
            //发布面包靴
            List<CrumbViewModel> breadCrumbs = new List<CrumbViewModel>();
            breadCrumbs.Add(new CrumbViewModel { Name = "系统管理" });
            breadCrumbs.Add(new CrumbViewModel { Name = "菜单管理", NavigateRegion = "MenuManageView" });
            _aggregator.GetEvent<BreadEvent>().Publish(breadCrumbs);
        }
        private void AddMenu(object obj)
        { 
            //蒙层显示
            _aggregator.GetEvent<MaskEvent>().Publish(true);
            _dialog.ShowDialog("AddEditMenuView", async callback =>
            {
                if (callback.Result == ButtonResult.OK)
                {
                    GetAllMenus();
                }

            });
            //蒙层显示
            _aggregator.GetEvent<MaskEvent>().Publish(false);
        }

        private void EditMenu(object obj)
        {
            int id = Convert.ToInt32(obj);

            DialogParameters parm = new DialogParameters();
            parm.Add("Key", id);
            //蒙层显示
            _aggregator.GetEvent<MaskEvent>().Publish(true);
            _dialog.ShowDialog("AddEditMenuView", parm, async callback =>
            {
                if (callback.Result == ButtonResult.OK)
                {
                    GetAllMenus();
                }

            });
            //蒙层
            _aggregator.GetEvent<MaskEvent>().Publish(false);
        }

        private void Delete(object obj)
        {
            int id = Convert.ToInt32(obj);
            MessageBoxResult boxResult = MessageBox.Show("确认删除此条数据?", "确认", MessageBoxButton.OKCancel, MessageBoxImage.Question);
            if (boxResult == MessageBoxResult.OK)
            {
                var del = _iMenuService.Delete(id);
                if (del)
                {
                    MessageBox.Show("删除成功!", "确认", MessageBoxButton.OK, MessageBoxImage.Information);
                    GetAllMenus();
                }

            }
        }
        private void GetAllMenus()
        {
            MenuInfoList.Clear();
            var menus = _iMenuService.QueryList();
            allMenuInfoList = _mapper.Map<List<Menu>, List<MenuDto>>(menus);
            foreach(var item in allMenuInfoList)
            {
                if(!string.IsNullOrEmpty(item.MenuIcon))
                {
                    item.MenuIcon = System.IO.Path.Combine(basePath, item.MenuIcon);
                }
            }
            //按照父菜单和子菜单来排序
            var menuParents=allMenuInfoList.FindAll(x=>x.ParentId==0)?.OrderBy(y => y.Index);
            foreach(var menu in menuParents)
            {
                MenuInfoList.Add(menu);
                //查找下面的子菜单
                var menuPs = allMenuInfoList?.FindAll(x => x.ParentId == menu.MenuId)?.OrderBy(y=>y.Index);
                foreach(var item in menuPs)
                {
                    MenuInfoList.Add(item);
                }

            }
          
        }

        /// <summary>
        /// 判断输入的数据
        /// </summary>
        /// <param name="input"></param>

        /// <summary>
        /// 获取PLC配置
        /// </summary>

        /// <summary>
        /// 查询
        /// </summary>
        /// <param name="obj"></param>
        private void Query(object obj)
        {
            var conMenus = (from a in allMenuInfoList
                            where (string.IsNullOrEmpty(MenuName) ? true : (a.MenuHeader.Contains(MenuName)))

                            select a).ToList();
            //默认显示的第一页

            MenuInfoList = new ObservableCollection<MenuDto>(conMenus);
        }

        #endregion
        #region 数据绑定
        private ObservableCollection<MenuDto> menuInfoList = new ObservableCollection<MenuDto>();
        public ObservableCollection<MenuDto> MenuInfoList
        {
            get { return menuInfoList; }
            set { menuInfoList = value; RaisePropertyChanged(); }
        }
        private string menuName = string.Empty;
        public string MenuName
        {
            get { return menuName; }
            set { menuName = value; RaisePropertyChanged(); }
        }

        #endregion

        #region 命令绑定
        public DelegateCommand OnLoadCommand { set; get; }
        public DelegateCommand<object> AddCommand { set; get; }
        public DelegateCommand<object> EditCommand { set; get; }
        public DelegateCommand<object> DeleteCommand { set; get; }
        public DelegateCommand<object> QueryCommand { set; get; }

        #endregion
    }
}
