﻿using AutoMapper;
using BizService;
using Microsoft.Extensions.Logging;
using Microsoft.VisualBasic.ApplicationServices;
using Model.Dto;
using Model.Entities;
using PLCTool.Common;
using PLCTool.Controls;
using PLCTool.Events;
using PLCTool.Models;
using Prism.Commands;
using Prism.Events;
using Prism.Mvvm;
using Prism.Services.Dialogs;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows;

namespace PLCTool.ViewModels.SystemManageViewModel
{
    public  class UserManageViewModel:BindableBase
    {
        private readonly IUserService _iUserService;
        private readonly IUserRoleService _iUserRoleService;
        private readonly IRoleService _iRoleService;
        private readonly ILogger _logger;
        private readonly IMapper _mapper;
        private readonly IDialogService _dialog;
        private readonly IEventAggregator _aggregator;
        private List<UserWithRoleModel> allUserInfoList = new List<UserWithRoleModel>();
        public UserManageViewModel(IUserService iUserService, IUserRoleService iUserRoleService, IRoleService iRoleService,IMapper mapper, ILogger logger, IDialogService dialog,IEventAggregator aggregator)
        {

            _iUserService = iUserService;
            _iUserRoleService = iUserRoleService;
            _iRoleService = iRoleService;
            _mapper = mapper;
            _logger = logger;
            _dialog = dialog;
            _aggregator = aggregator;
            AddCommand = new DelegateCommand<object>(AddUser);
            EditCommand = new DelegateCommand<object>(EditUser);
            DeleteCommand = new DelegateCommand<object>(Delete);
            OnLoadCommand = new DelegateCommand(OnLoad);
            QueryCommand = new DelegateCommand<object>(Query);
            AuthoDeviceCommand = new DelegateCommand<object>(AuthDevice);
        }

      




        #region 私有方法
        private void OnLoad()
        {
            GetAllUsers();
            //发布面包靴
            List<CrumbViewModel> breadCrumbs = new List<CrumbViewModel>();
            breadCrumbs.Add(new CrumbViewModel { Name = "系统管理" });
            breadCrumbs.Add(new CrumbViewModel { Name = "用户管理", NavigateRegion = "UserManageView" });
            _aggregator.GetEvent<BreadEvent>().Publish(breadCrumbs);
        }
        private void AddUser(object obj)
        {
            _dialog.ShowDialog("AddEditUserView", async callback =>
            {
                if (callback.Result == ButtonResult.OK)
                {
                    GetAllUsers();
                }

            });
        }
        private void AuthDevice(object obj)
        {
            int id = Convert.ToInt32(obj);

            DialogParameters parm = new DialogParameters();
            parm.Add("Key", id);
            //蒙层显示
            _aggregator.GetEvent<MaskEvent>().Publish(true);
            _dialog.ShowDialog("UserWithDeviceView", parm, async callback =>
            {
                if (callback.Result == ButtonResult.OK)
                {
               
                }

            });
            _aggregator.GetEvent<MaskEvent>().Publish(false);
        }

        private void EditUser(object obj)
        {
            int id = Convert.ToInt32(obj);
            var user = _iUserService.Find(id);
            if ((user != null) && (user.UserName == "admin"))
            {
                MessageBox.Show("管理员用户无法编辑！", "确认", MessageBoxButton.OK, MessageBoxImage.Information);
                return;
            }
            DialogParameters parm = new DialogParameters();
            parm.Add("Key", id);
            //蒙层显示
            _aggregator.GetEvent<MaskEvent>().Publish(true);
            _dialog.ShowDialog("AddEditUserView", parm, async callback =>
            {
                if (callback.Result == ButtonResult.OK)
                {
                    GetAllUsers();
                }

            });
            _aggregator.GetEvent<MaskEvent>().Publish(false);
        }

        private void Delete(object obj)
        {
            int id = Convert.ToInt32(obj);
            var user = _iUserService.Find(id);
            if ((user != null)&&(user.UserName=="admin"))
            {
                MessageBox.Show("管理员用户无法删除！", "确认", MessageBoxButton.OK, MessageBoxImage.Information);
                return;
            }
            if (id == Appsession.UserId)
            {
                MessageBox.Show("此用户与当前登录用户相同，无法删除！", "确认", MessageBoxButton.OK, MessageBoxImage.Information);
                return;
            }
            MessageBoxResult boxResult = MessageBox.Show("确认删除此条数据?", "确认", MessageBoxButton.OKCancel, MessageBoxImage.Question);
            if (boxResult == MessageBoxResult.OK)
            {
                var del = _iUserService.Delete(id);
                if (del)
                {
                    MessageBox.Show("删除成功!", "确认", MessageBoxButton.OK, MessageBoxImage.Information);
                    GetAllUsers();
                }

            }
        }
        private  void GetAllUsers()
        {
            allUserInfoList.Clear();
            UserInfoList.Clear();
            var users = _iUserService.QueryList();
            foreach(var user in users)
            {
                var userRo= _iUserRoleService.FindByUserID(user.Id);
                if(userRo!=null)
                {
                    int roleId= (int)userRo.RoleId;
                    string roleName = _iRoleService.Find(roleId)?.Name;
                    UserWithRoleModel userRole = new UserWithRoleModel()
                    {
                        Id=user.Id,
                        UserName=user.UserName,
                        RoleName=roleName,
                        Description=user.Description,

                    };
                    allUserInfoList.Add(userRole);
                    UserInfoList.Add(userRole);
                }
             

            }
          
        }

        /// <summary>
        /// 判断输入的数据
        /// </summary>
        /// <param name="input"></param>

        /// <summary>
        /// 获取PLC配置
        /// </summary>

        /// <summary>
        /// 查询
        /// </summary>
        /// <param name="obj"></param>
        private void Query(object obj)
        {
            var conUsers = (from a in allUserInfoList
                            where (string.IsNullOrEmpty(UserName) ? true : (a.UserName.Contains(UserName)))
                               select a).ToList();
            //默认显示的第一页

            UserInfoList = new ObservableCollection<UserWithRoleModel>(conUsers);
        }

        #endregion
        #region 数据绑定
        private ObservableCollection<UserWithRoleModel> userInfoList = new ObservableCollection<UserWithRoleModel>();
        public ObservableCollection<UserWithRoleModel> UserInfoList
        {
            get { return userInfoList; }
            set { userInfoList = value; RaisePropertyChanged(); }
        }
        private string userName = string.Empty;
        public string UserName
        {
            get { return userName; }
            set { userName = value; RaisePropertyChanged(); }
        }

        #endregion

        #region 命令绑定
        public DelegateCommand OnLoadCommand { set; get; }
        public DelegateCommand<object> AddCommand { set; get; }
        public DelegateCommand<object> EditCommand { set; get; }
        public DelegateCommand<object> DeleteCommand { set; get; }
        public DelegateCommand<object> QueryCommand { set; get; }
        public DelegateCommand<object> AuthoDeviceCommand { set; get; }
        

        #endregion
    }
}
